{% extends base.html %}

{% block content %}
<form class="signin layui-form" action="/signin" method="POST" href="/">
  <div class="layui-form-item">
    <label class="layui-form-label">用户名</label>
    <div class="layui-input-block">
      <input type="text" name="username" required placeholder="请输入用户名" class="layui-input">
    </div>
  </div>
  <div class="layui-form-item">
    <label class="layui-form-label">密码</label>
    <div class="layui-input-block">
      <input type="password" name="password" required placeholder="请输入密码" class="layui-input">
    </div>
  </div>
  <div class="layui-form-item" pane="">
    <div class="layui-input-block">
      <input type="checkbox" name="remember" lay-skin="primary" title="记住我" checked="checked">
      <a class='reset' href="/reset">忘记密码</a>
    </div>
  </div>
  <div class="layui-form-item">
    <div class="layui-input-block">
      <button class="layui-btn" lay-submit lay-filter="default">立即提交</button>
    </div>
  </div>
</form>

{% end %}
